﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OrbitalSurveyPlus
{
    public class ModuleOrbitalSurveyorPlus : PartModule, IAnimatedModule, IResourceConsumer
    {
        static readonly string STATUS_OFF = "standby";
        static readonly string STATUS_ON = "scanning";
        static readonly string STATUS_TOO_LOW = "too low";
        static readonly string STATUS_TOO_HIGH = "too high";
        static readonly string STATUS_NO_POWER = "not enough ElectricCharge";

        [KSPField(guiActive = false, guiName = "Status")]
        string scanStatus = STATUS_OFF;

        [KSPField(guiActive = false, guiName = "Min Altitude")]
        string minAltitude = "";

        [KSPField(guiActive = false, guiName = "Max Altitude")]
        string maxAltitude = "";

        [KSPField(guiActive = false, guiName = "Surface Scanned")]
        string percentScanned = "0%";

        [KSPField(isPersistant = true)]
        public bool scannerOn = false;

        [KSPField(isPersistant = true)]
        public bool perpetualScan = false;

        //Module Tweakable Fields
        [KSPField]
        public float ElectricDrain = 0.75f;
        [KSPField(isPersistant = true)]
        public int ScanRadius = 10;
        [KSPField]
        public int SciBonus = 10;

        private bool activated;
        private List<ModuleOrbitalSurveyor> moduleStockSurveyor;
        private PartResourceDefinition resEC = PartResourceLibrary.Instance.GetDefinition("ElectricCharge");
        private double lastUpdate;
        private double lastScan;
        private bool freshStart;

        private CelestialBody currentBody;

        //IAnimateModule Implement
        public void DisableModule()
        {
            activated = false;
            scannerOn = false;
            
            SetUIVisibility();
        }

        public void EnableModule()
        {
            if (OSPGlobal.ExtendedSurvey)
            { 
                activated = true;
                DisableStockSurveyor();
            }
            lastUpdate = Planetarium.GetUniversalTime();
            lastScan = 0;
            SetAltitudeHelp();
            SetUIVisibility();
        }

        public bool IsSituationValid()
        {
            return true;
        }

        public bool ModuleIsActive()
        {
            return activated;
        }

        //IResourceConsumer Implement
        public List<PartResourceDefinition> GetConsumedResources()
        {
            PartResourceDefinition consumedRes = PartResourceLibrary.Instance.GetDefinition("ElectricCharge");
            List<PartResourceDefinition> list = new List<PartResourceDefinition>();
            list.Add(consumedRes);
            return list;
        }

        //Unity Functions
        public override void OnStart(StartState state)
        {
            base.OnStart(state);

            lastUpdate = Planetarium.GetUniversalTime();
            freshStart = true;

            moduleStockSurveyor = part.FindModulesImplementing<ModuleOrbitalSurveyor>();
        }

        public void Update()
        {
            if (freshStart)
            {
                freshStart = false;
                currentBody = vessel.mainBody;
                SetAltitudeHelp();
                SetUIVisibility();
                if (OSPGlobal.ExtendedSurvey) DisableStockSurveyor();
            }

            if (vessel.mainBody != currentBody)
            {
                currentBody = vessel.mainBody;
                SetAltitudeHelp();
            }
        }

        public override void OnFixedUpdate()
        {
            base.OnFixedUpdate();

            if (!OSPGlobal.ExtendedSurvey) return;

            perpetualScan = false;
            if (activated)
            {
                scanStatus = STATUS_OFF;

                if (scannerOn)
                {
                    //get elapsed time since last update
                    double timeElapsed = 0;
                    double ut = Planetarium.GetUniversalTime();
                    timeElapsed = ut - lastUpdate;
                    lastUpdate = ut;

                    //if conditions met, proceed with scan
                    if (CheckConditions() && timeElapsed > 0) //CheckConditions() will also chang scanStatus to reflect problems
                    {
                        //check electric charge
                        double drain = ElectricDrain * timeElapsed;
                        Vessel.ActiveResource ar = vessel.GetActiveResource(resEC);

                        if (ar.amount >= ElectricDrain)
                        {
                            //we are scanning!
                            scanStatus = STATUS_ON;

                            if (lastScan + OSPGlobal.TimeBetweenScans <= ut)
                            {
                                lastScan = ut;

                                //if it isn't counted as "scanned" yet, go ahead and count it
                                ResourceMap.Instance.UnlockPlanet(vessel.mainBody.flightGlobalsIndex);

                                //update scan
                                OSPScenario.UpdateScanData(vessel.mainBody, true, vessel.longitude, vessel.latitude, ScanRadius);

                                //update percent scanned
                                UpdateScanInfo();
                            }

                            //for now, assume perpetual scanning
                            perpetualScan = true;
                        }
                        else
                        {
                            //not enough EC
                            scanStatus = STATUS_NO_POWER;
                        }

                        part.RequestResource(resEC.id, drain);
                    }
                }
            }
        }

        //Scan Buttons and Context Menu
        [KSPEvent(guiName = "Start Survey", guiActive = true)]
        public void startSurvey()
        {
            scannerOn = true;
            lastUpdate = Planetarium.GetUniversalTime();
            SetUIVisibility();
        }

        [KSPEvent(guiName = "Stop Survey", guiActive = true)]
        public void stopSurvey()
        {
            scannerOn = false;
            SetUIVisibility();
        }

        private void SetUIVisibility()
        {
            Fields["scanStatus"].guiActive = false;
            Fields["minAltitude"].guiActive = false;
            Fields["maxAltitude"].guiActive = false;
            Fields["percentScanned"].guiActive = false;
            
            Events["startSurvey"].active = false;
            Events["stopSurvey"].active = false;

            if (OSPGlobal.ExtendedSurvey && activated)
            {
                Fields["scanStatus"].guiActive = true;
                Fields["minAltitude"].guiActive = true;
                Fields["maxAltitude"].guiActive = true;
                Fields["percentScanned"].guiActive = true;

                if (scannerOn)
                {
                    Events["stopSurvey"].active = true;
                }
                else
                {
                    Events["startSurvey"].active = true;
                }
            }
        }

        private void SetAltitudeHelp()
        {
            minAltitude = OSPGlobal.DistanceToString(OSPGlobal.ScanMinimumAltitude(vessel.mainBody));
            maxAltitude = OSPGlobal.DistanceToString(OSPGlobal.ScanMaximumAltitude(vessel.mainBody));
            Fields["minAltitude"].guiName = "Min Altitude (" + vessel.mainBody.name + ")";
            Fields["maxAltitude"].guiName = "Max Altitude (" + vessel.mainBody.name + ")";
        }

        private void UpdateScanInfo()
        {
            double scanPct = OSPScenario.GetScanPercent(vessel.mainBody);

            //update scan percent field
            int pct = (int)Math.Floor(scanPct*100);
            percentScanned = pct.ToString() + "%";
        }

        //Other Stuff
        private bool CheckConditions()
        {
            CelestialBody body = vessel.mainBody;
            if (body == null)
            {
                return false;
            }

            double minAltitude = OSPGlobal.ScanMinimumAltitude(body);
            double maxAltitude = OSPGlobal.ScanMaximumAltitude(body);

            if (vessel.altitude < minAltitude)
            {
                scanStatus = STATUS_TOO_LOW;
                return false;
            }

            if (vessel.altitude > maxAltitude)
            {
                scanStatus = STATUS_TOO_HIGH;
                return false;
            }

            return true;
        }

        private void DisableStockSurveyor()
        {
            if (moduleStockSurveyor != null)
            {
                foreach (ModuleOrbitalSurveyor m in moduleStockSurveyor)
                    m.DisableModule();
            }
        }

        public override string GetInfo()
        {
            //show electric charge info 
            String s = base.GetInfo();
            s += "\n\n<b><color=#99ff00ff>Requires (while scanning):</color></b>";
            s += "\nElectric Charge: " + ElectricDrain + "/sec.";
            return s;
        }
    }
}

