﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;

namespace OrbitalSurveyPlus
{
    class OSPGlobal
    {
        //------------ VERSION STUFF -----------------------------------------------------------------------------
        public const string OSP_TITLE = "Orbital Survey Plus";
        public const string OSP_AUTHOR = "Chase Barnes (Wheffle)";
        public const string OSP_EMAIL = "altoid287@gmail.com";
        public const int VERSION_MAJOR = 2;
        public const int VERSION_MINOR = 0;
        public const int VERSION_PATCH = 2;
        public const string VERSION_KSP = "1.1";
        public static readonly DateTime VERSION_DATE = new DateTime(2016, 4, 29);

        public static readonly string VERSION_STRING = 
            VERSION_PATCH > 0 ? 
            VERSION_MAJOR + "." + VERSION_MINOR + "." + VERSION_PATCH : 
            VERSION_MAJOR + "." + VERSION_MINOR;
        //--------------------------------------------------------------------------------------------------------

        //NODE STUFF
        public const string OSPScanDataNodeName = "OSPScanData";
        public const string OSPScanDataValueName = "data";
        public const string OSPSciencePctValueName = "sciencePct";

        //SETTINGS STUFF
        public const string SettingsPath = "GameData/OrbitalSurveyPlus/settings.cfg";
        public const string InfoPath = "GameData/OrbitalSurveyPlus/info.txt";

        public const string sExtendedSurvey = "ExtendedSurvey";
        public const bool ExtendedSurveyDefault = true;
        public static bool ExtendedSurvey
        {
            get;
            set;
        } = ExtendedSurveyDefault;

        public const string sBiomeMapRequiresScan = "BiomeMapRequiresScan";
        public const bool BiomeMapRequiresScanDefault = true;
        public static bool BiomeMapRequiresScan
        {
            get;
            set;
        } = BiomeMapRequiresScanDefault;

        public const string sBackgroundScan = "BackgroundScan";
        public const bool BackgroundScanDefault = true;
        public static bool BackgroundScan
        {
            get;
            set;
        } = BackgroundScanDefault;

        public const string sMinAltitudeFactor = "MinAltitudeFactor";
        public const double MinAltitudeFactorDefault = 0.1;
        public static double MinAltitudeFactor
        {
            get;
            set;
        } = MinAltitudeFactorDefault;

        public const string sMinAltitudeAbsolute = "MinAltitudeAbsolute";
        public const double MinAltitudeAbsoluteDefault = 25000;
        public static double MinAltitudeAbsolute
        {
            get;
            set;
        } = MinAltitudeAbsoluteDefault;

        public const string sMaxAltitudeFactor = "MaxAltitudeFactor";
        public const double MaxAltitudeFactorDefault = 5;
        public static double MaxAltitudeFactor
        {
            get;
            set;
        } = MaxAltitudeFactorDefault;

        public const string sMaxAltitudeAbsolute = "MaxAltitudeAbsolute";
        public const double MaxAltitudeAbsoluteDefault = 15000000;
        public static double MaxAltitudeAbsolute
        {
            get;
            set;
        } = MaxAltitudeAbsoluteDefault;

        public const string sTimeBetweenScans = "TimeBetweenScans";
        public const double TimeBetweenScansDefault = 1;
        public static double TimeBetweenScans
        {
            get;
            set;
        } = TimeBetweenScansDefault;

        public static void ResetSettingsToDefault()
        {
            ExtendedSurvey = ExtendedSurveyDefault;
            BiomeMapRequiresScan = BiomeMapRequiresScanDefault;
            BackgroundScan = BackgroundScanDefault;
            MinAltitudeFactor = MinAltitudeFactorDefault;
            MinAltitudeAbsolute = MinAltitudeAbsoluteDefault;
            MaxAltitudeFactor = MaxAltitudeFactorDefault;
            MaxAltitudeAbsolute = MaxAltitudeAbsoluteDefault;
            TimeBetweenScans = TimeBetweenScansDefault;
        }

        public static int longitudeToX(double lon, int width)
        {
            //lon starts out between -180 and 180
            //shift to between 0 and 360
            lon += 270;

            //double check for weird numbers from KSP engine
            while (lon < 0) lon += 360;
            while (lon > 360) lon -= 360;

            //find pixel based on scale
            double scale = 1 - (lon / 360);
            return (int)Math.Round(scale * width);
        }

        public static int latitudeToY(double lat, int height)
        {
            //lat starts out between -90 and 90
            //shift to between 0 and 180
            lat += 90;

            //find pixel based on scale
            double scale = lat / 180;
            return (int)Math.Round(scale * height);
        }

        public static bool withinDistance(double x1, double y1, double x2, double y2, double dist)
        {
            return ((x2 - x1) * (x2 - x1)) + ((y2 - y1) * (y2 - y1)) <= dist * dist;
        }

        public static void Log(String log)
        {
            PDebug.Log("[OrbitalSurveyPlus] " + log);
        }

        public static double ScanMinimumAltitude(CelestialBody body)
        {
            double minAltitude = Math.Max(body.Radius * MinAltitudeFactor, MinAltitudeAbsolute);
            return minAltitude;
        }

        public static double ScanMaximumAltitude(CelestialBody body)
        {
            double maxAltitude = Math.Min(body.Radius * MaxAltitudeFactor, MaxAltitudeAbsolute);
            return maxAltitude;
        }

        public static string DistanceToString(double dist)
        {
            if (dist < 1000)        return String.Format("{0:0.00}m", dist);
            if (dist < 1000000)     return String.Format("{0:0.00}km", dist/1000);
            if (dist < 1000000000)  return String.Format("{0:0.00}Mm", dist / 1000000);
            return String.Format("{0:0.00}Gm", dist / 1000000000);
        }

        public static Color32[] GetPixels32FromBiomeMap(CelestialBody body, out int width, out int height)
        {
            CBAttributeMapSO bm = body.BiomeMap;
            width = bm.Width;
            height = bm.Height;
            Color32[] pixels = new Color32[width * height];
            for (int j = 0; j < height; j++)
            {
                for (int i = 0; i < width; i++)
                {
                    pixels[(j * width) + i] = bm.GetPixelColor32(i, j);
                }
            }
            return pixels;
        }

        public static double GetVesselResourceProduction(Vessel v, string resName)
        {
            double production = 0;

            //Check Electric Generators
            List<ModuleGenerator> generators = v.FindPartModulesImplementing<ModuleGenerator>();
            foreach (ModuleGenerator m in generators)
            {
                if (m.generatorIsActive)
                {
                    foreach(ModuleResource r in m.outputList)
                    {
                        if (r.name == resName) production += r.rate;
                    }
                }
            }

            //Check Solar Panels
            List<ModuleDeployableSolarPanel> solarPanels = v.FindPartModulesImplementing<ModuleDeployableSolarPanel>();
            foreach (ModuleDeployableSolarPanel m in solarPanels)
            {
                if (m.panelState == ModuleDeployableSolarPanel.panelStates.EXTENDED)
                {
                    if (m.resourceName == resName) production += m.chargeRate;
                }
            }

            //Check Converters
            List<ModuleResourceConverter> converters = v.FindPartModulesImplementing<ModuleResourceConverter>();
            foreach (ModuleResourceConverter m in converters)
            {
                if (m.ModuleIsActive())
                {
                    foreach (ResourceRatio r in m.outputList)
                    {
                        if (r.ResourceName == resName) production += r.Ratio;
                    }
                }
            }

            return production;
        }
    }
}
