﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OrbitalSurveyPlus
{
    class ScanDataBody
    {
        private bool[,] data;

        public int Width
        {
            get;
            private set;
        }

        public int Height
        {
            get;
            private set;
        }

        public double SciencePct
        {
            get;
            set;
        }

        

        public ScanDataBody(CelestialBody body)
        {
            //represent the scan data as a 2D boolean array (false = shrouded, true = scanned)
            //instead of representing each pixel, data is divided into sections so that save/load won't take 15 minutes
            //basically, it's a low-resolution represenation of what is scanned and what isn't

            //create a data array based on body's circumference (2*pi*r)
            Height = (int)((2 * Math.PI * body.Radius) / 25000);
            Width = Height * 2;
            data = new bool[Width, Height];

            //fill it with false to avoid juggling nulls later on
            for (int j = 0; j < Height; j++)
            {
                for (int i = 0; i < Width; i++)
                {
                    data[i, j] = false;
                }
            }

            //set initial values for scan and science percent
            SciencePct = 0;
        }

        public void Load(string serializedHex, double sciencePct)
        {


            //------Legacy function so that old saves won't lose their data ----------------
            if (serializedHex.Length == Width * Height)
            {
                //read string as a flat array
                for (int j = 0; j < Height; j++)
                {
                    for (int i = 0; i < Width; i++)
                    {
                        if (serializedHex[i + (j * Width)] == '1')
                        {
                            data[i, j] = true;
                        }
                        else
                        {
                            data[i, j] = false;
                        }
                    }
                }
                return;
            }
            //-------------------------------------------------------------------------------

            //convert seralized hex string to flat bool array
            bool[] flatBool = new bool[Width * Height];
            int pos = 0;
            for (int i = 0; i < serializedHex.Length; i++)
            {
                string bin = HexToBinary(serializedHex[i]);
                for (int j = 0; j < bin.Length; j++)
                {
                    flatBool[pos] = bin[j] == '1' ? true : false;
                    pos++;
                    if (pos >= flatBool.Length) break;
                }
                if (pos >= flatBool.Length) break;
            }

            //convert flat array to 2D data array
            for (int j = 0; j < Height; j++)
            {
                for (int i = 0; i < Width; i++)
                {
                    data[i, j] = flatBool[(j * Width) + i];
                }
            }

            //set science percent
            sciencePct = SciencePct;
        }

        public string SerializeToString()
        {
            /*
            String serialized = "";
            for (int j = 0; j < Height; j++)
            {
                for (int i = 0; i < Width; i++)
                {
                    serialized += data[i, j] ? "1" : "0";
                }
            }
            return serialized;
            */

            string serializedHex = "";
            string bin = "";
            for (int j = 0; j < Height; j++)
            {
                for (int i = 0; i < Width; i++)
                {
                    bin += data[i, j] ? "1" : "0";

                    if (bin.Length > 3)
                    {
                        serializedHex += BinaryToHex(bin);
                        bin = "";
                    }
                }
            }

            if (bin.Length > 0)
            {
                while (bin.Length < 3) bin += "0";
                serializedHex += BinaryToHex(bin);
            }

            return serializedHex;
        }

        private char BinaryToHex(string bin)
        {
            switch(bin)
            {
                case "0000": return '0';
                case "0001": return '1';
                case "0010": return '2';
                case "0011": return '3';
                case "0100": return '4';
                case "0101": return '5';
                case "0110": return '6';
                case "0111": return '7';
                case "1000": return '8';
                case "1001": return '9';
                case "1010": return 'A';
                case "1011": return 'B';
                case "1100": return 'C';
                case "1101": return 'D';
                case "1110": return 'E';
                case "1111": return 'F';
            }
            OSPGlobal.Log("ERROR: conversion from binary string to hex string got weird input!");
            return '0';
        }

        private string HexToBinary(char hex)
        {
            switch(hex)
            {
                case '0': return "0000";
                case '1': return "0001";
                case '2': return "0010";
                case '3': return "0011";
                case '4': return "0100";
                case '5': return "0101";
                case '6': return "0110";
                case '7': return "0111";
                case '8': return "1000";
                case '9': return "1001";
                case 'A': case 'a':  return "1010";
                case 'B': case 'b':  return "1011";
                case 'C': case 'c':  return "1100";
                case 'D': case 'd':  return "1101";
                case 'E': case 'e':  return "1110";
                case 'F': case 'f':  return "1111";
            }
            OSPGlobal.Log("ERROR: conversion from hex string to binary string on scan data serializaion got weird input!");
            return "0000";
        }

        public void UpdateScanData(bool scanned, double lon, double lat, int radius)
        {
            //find the data point that matches the given lon and lat
            int point_x = OSPGlobal.longitudeToX(lon, Width);
            int point_y = OSPGlobal.latitudeToY(lat, Height);

            int half_width = Width / 2;
            int half_height = Height / 2;

            //calculate stretch offset (the scan area tends to get skinnier as it nears the poles, this helps mitigate that)
            double stretchFactor = Math.Abs((double)point_y - (double)half_height) / (double)half_height;
            int stretchOffset = (int)Math.Round((double)radius * stretchFactor);
            int point_x_west = point_x - stretchOffset;
            int point_x_east = point_x + stretchOffset;

            /*
                the stretch offset basically creates two more circles of scan east and west of the
                actual scan point to try to mitigate width shrinkage as you approach the poles
            */

            //update data point and surrounding data points in radius
            for (int j = -radius; j <= radius; j++)
            {
                for (int i = -radius-stretchOffset; i <= radius+stretchOffset; i++)
                {
                    int x = point_x + i;
                    int y = point_y + j;

                    //check total distance to see if it's within radius (or stretch points)
                    if (OSPGlobal.withinDistance(point_x, point_y, x, y, radius) ||
                        OSPGlobal.withinDistance(point_x_west, point_y, x, y, radius) ||
                        OSPGlobal.withinDistance(point_x_east, point_y, x, y, radius))
                    {
                        //wrap east-west
                        if (x < 0)
                        {
                            x = Width + x;
                        }
                        else if (x >= Width)
                        {
                            x -= Width;
                        }

                        //wrap over poles
                        if (y < 0)
                        {
                            y = 0 - y;
                            x += half_width;
                            if (x >= Width) x -= Width;
                        }
                        else if (y >= Height)
                        {
                            y = Height + (Height - (y+1));
                            x += half_width;
                            if (x >= Width) x -= Width;
                        }

                        //sanity check
                        if (x >= 0 && x < Width && y >= 0 && y < Height)
                        {
                            //update point
                            data[x, y] = scanned;
                        }
                        else
                        {
                            OSPGlobal.Log(String.Format("ERROR: Index out of bounds for UpdateScanData! Width={0}, Height={1}, x={2}, y={3}", Width, Height, x, y));
                        }
                    }
                }
            }
        }

        public double GetScanPercent()
        {
            double total = Width * Height;
            double scanned = 0;

            for (int j = 0; j < Height; j++)
            {
                for (int i = 0; i < Width; i++)
                {
                    if (data[i, j]) scanned += 1;
                }
            }

            return scanned / total;
        }

        public bool GetScanPoint(int x, int y)
        {
            return data[x, y];
        }

    }
}
