﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace OrbitalSurveyPlus
{
    class ScanDataUniverse
    {
        private Dictionary<int, ScanDataBody> bodyScanData;

        public ScanDataUniverse()
        {
            bodyScanData = new Dictionary<int, ScanDataBody>();
        }

        public void Clear()
        {
            bodyScanData.Clear();
        }

        public void Load(ConfigNode node)
        {
            string nodeName = OSPGlobal.OSPScanDataNodeName;
            string dataName = OSPGlobal.OSPScanDataValueName;
            string sciencePctName = OSPGlobal.OSPSciencePctValueName;

            //clear current data
            Clear();

            //check to see if node exists
            ConfigNode scanDataNode = node.GetNode(nodeName);
            if (scanDataNode == null)
            {
                return;
            }

            //node exists, so load stored data for bodies that have been scanned and saved
            foreach (CelestialBody body in FlightGlobals.Bodies)
            {
                //get body attributes
                String bodyName = body.GetName();
                int bodyId = body.flightGlobalsIndex;

                //retrieve scan data from node, if it exists
                ConfigNode bodyNode = scanDataNode.GetNode(bodyName);

                if (bodyNode != null)
                {
                    ScanDataBody scanData = new ScanDataBody(body);

                    int width = scanData.Width;
                    int height = scanData.Height;

                    if (bodyNode.HasValue(dataName))
                    {
                        string serialized = bodyNode.GetValue(dataName);
                        double sciencePct = 0;

                        if (bodyNode.HasValue(sciencePctName))
                        {
                            string sciencePctValue = bodyNode.GetValue(sciencePctName);
                            sciencePct = double.Parse(sciencePctValue);
                        }

                        scanData.Load(serialized, sciencePct);
                        bodyScanData.Add(bodyId, scanData);
                    }
                    else
                    {
                        OSPGlobal.Log("ERROR: Scan data for " + body.name + " could not be loaded! (missing node value)");
                    }
                }
            }
        }

        public void Save(ConfigNode node)
        {
            string nodeName = OSPGlobal.OSPScanDataNodeName;
            string dataValueName = OSPGlobal.OSPScanDataValueName;
            string sciencePctValueName = OSPGlobal.OSPSciencePctValueName;

            ConfigNode scanDataNode = new ConfigNode(nodeName);

            foreach (CelestialBody body in FlightGlobals.Bodies)
            {
                //get body attributes
                String bodyName = body.GetName();
                int bodyId = body.flightGlobalsIndex;

                //get scan data
                ScanDataBody scanData;
                bool dataExists = bodyScanData.TryGetValue(bodyId, out scanData);

                //save data in new node for this body, only if data exists
                if (dataExists)
                {
                    ConfigNode bodyNode = new ConfigNode(bodyName);

                    int width = scanData.Width;
                    int height = scanData.Height;

                    //save array to memory as a string of 1's and 0's (binary)
                    String serialized = scanData.SerializeToString();
                    bodyNode.AddValue(dataValueName, serialized);

                    //save science percent
                    bodyNode.AddValue(sciencePctValueName, scanData.SciencePct);

                    //attach to parent node
                    scanDataNode.AddNode(bodyNode);
                }
            }

            node.AddNode(scanDataNode);
        }

        public ScanDataBody GetBodyScanData(int bodyIndex)
        {
            if (bodyScanData.ContainsKey(bodyIndex))
            {
                return bodyScanData[bodyIndex];
            }
            return null;
        }

        public void UpdateScanData(CelestialBody body, bool scanned, double lon, double lat, int radius)
        {
            //get existing data, or create new one if none exist
            ScanDataBody scanData = GetBodyScanData(body.flightGlobalsIndex);
            if (scanData == null)
            {
                scanData = new ScanDataBody(body);
                bodyScanData.Add(body.flightGlobalsIndex, scanData);
            }

            scanData.UpdateScanData(scanned, lon, lat, radius);
        }
    }
}
